package codeline;

import processor.ControlFlow;
import processor.Memory;
import processor.Memory.ReadOnlyMemory;
import processor.Memory.Visibility;

/**
 * sets the value of a previously declared variable.
 * if the variable has not been declared, nothing happens.
 * @author kolja
 *
 * @param <T> the type of the variable
 */
public abstract class SetVariable <T> extends CodeLine {

    private String name;
    
    public SetVariable( String name )
    {
        this.name = name;
    }

    @Override
    public ControlFlow runForward(Memory m) {
        
        T oldVal = m.read( name, Visibility.LOCAL );
        m.write( name, value( m.createReadOnlyMemory() ), Visibility.LOCAL );
        actions.push( (Memory mem) -> {
            mem.write( name, oldVal, Visibility.LOCAL );
        });
        return new ControlFlow( ControlFlow.STEP_OVER );
    }

    /**
     * returns the value that will be stored in the variable
     * @param m the memory in which the variables are stored
     * @return the value
     */
    protected abstract T value( ReadOnlyMemory m );
}
